@@ -1,10 +1,10 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999, 2000, 2001, 2005 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2001, 2005, 2007 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,19 +13,15 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#define _GNU_SOURCE 1
-
-#include "config.h"
+#include <config.h>
 
 #include <parted/parted.h>
 #include <parted/debug.h>
 #include <parted/gnu.h>
 
-#include <string.h>
 #include <errno.h>
 #include <hurd.h>
 #include <hurd/fs.h>
@@ -397,8 +393,8 @@ gnu_refresh_close (PedDevice* dev)
 }
 
 static int
-gnu_read (PedDevice* dev, void* user_buffer,
-	  PedSector device_start, PedSector count)
+gnu_read (const PedDevice* dev, void* user_buffer, PedSector device_start,
+          PedSector count)
 {
 	GNUSpecific*		arch_specific = GNU_SPECIFIC (dev);
 	error_t			err;
@@ -657,7 +653,7 @@ doggy_first_block_write:
 
 	/* We are now left with (strictly) less then a store block to write
 	   to disk.  Thus, we read the block, overlay the buffer and flush.  */
-	PED_ASSERT (count * PED_SECTOR_SIZE
+	PED_ASSERT (count * PED_SECTOR_SIZE_DEFAULT
 			< arch_specific->store->block_size, return 0);
 
 doggy_last_block_read:
