@@ -1,10 +1,10 @@
 /*
     ext2.h -- ext2 header
-    Copyright (C) 1998-2000 Free Software Foundation, Inc.
+    Copyright (C) 1998-2000, 2007 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,15 +13,12 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef _EXT2_H
 #define _EXT2_H
 
-#include <config.h>
-
 #include <parted/parted.h>
 #include <parted/debug.h>
 #include <sys/types.h>
