@@ -5,7 +5,7 @@
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
     Contributor:  Phil Knirsch <phil@redhat.de>
                   Harald Hoyer <harald@redhat.de>
@@ -47,9 +46,6 @@
 #  define _(String) (String)
 #endif /* ENABLE_NLS */
 
-/* S390 */
-#if defined (__s390__) || defined (__s390x__)
-
 #define PARTITION_LINUX_SWAP 0x82
 #define PARTITION_LINUX 0x83
 #define PARTITION_LINUX_EXT 0x85
@@ -220,8 +216,9 @@ dasd_probe (const PedDevice *dev)
 	struct fdasd_anchor anchor;
 
 	PED_ASSERT(dev != NULL, return 0);
-	PED_ASSERT((dev->type == PED_DEVICE_DASD
-			   || dev->type == PED_DEVICE_VIODASD), return 0);
+
+	if (!(dev->type == PED_DEVICE_DASD || dev->type == PED_DEVICE_VIODASD))
+		return 0;
 
 	arch_specific = LINUX_SPECIFIC(dev);
 
@@ -272,6 +269,7 @@ dasd_read (PedDisk* disk)
 	char str[20];
 	PedDevice* dev;
 	PedPartition* part;
+	PedFileSystemType *fs;
 	PedSector start, end;
 	PedConstraint* constraint_exact;
 	partition_info_t *p;
@@ -382,19 +380,24 @@ dasd_read (PedDisk* disk)
 
 		dasd_data = part->disk_specific;
 
-		if (strncmp(PART_TYPE_RAID, str, 6) == 0)
+		if ((strncmp(PART_TYPE_RAID, str, 6) == 0) &&
+		    (ped_file_system_probe(&part->geom) == NULL))
 			ped_partition_set_flag(part, PED_PARTITION_RAID, 1);
 		else
 			ped_partition_set_flag(part, PED_PARTITION_RAID, 0);
 
-		if (strncmp(PART_TYPE_LVM, str, 6) == 0)
+		if ((strncmp(PART_TYPE_LVM, str, 6) == 0) &&
+		    (ped_file_system_probe(&part->geom) == NULL))
 			ped_partition_set_flag(part, PED_PARTITION_LVM, 1);
 		else
 			ped_partition_set_flag(part, PED_PARTITION_LVM, 0);
 
 		if (strncmp(PART_TYPE_SWAP, str, 6) == 0) {
-			dasd_data->system = PARTITION_LINUX_SWAP;
-			PDEBUG;
+			fs = ped_file_system_probe(&part->geom);
+			if (strncmp(fs->name, "linux-swap", 10) == 0) {
+				dasd_data->system = PARTITION_LINUX_SWAP;
+				PDEBUG;
+			}
 		}
 
 		vtoc_ebcdic_enc(p->f1->DS1DSNAM, p->f1->DS1DSNAM, 44);
@@ -865,5 +868,3 @@ error:
 	ped_constraint_destroy (constraint_any);
 	return 0;
 }
-
-#endif /* S390 */
