@@ -2,11 +2,11 @@
 
     libparted - a library for manipulating disk partitions
     disk_amiga.c - libparted module to manipulate amiga RDB partition tables.
-    Copyright (C) 2000, 2001, 2004 Free Software Foundation, Inc.
+    Copyright (C) 2000, 2001, 2004, 2007 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -15,20 +15,21 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
     Contributor:  Sven Luther <luther@debian.org>
 */
 
-#include "config.h"
-
-#include <string.h>
+#include <config.h>
 
 #include <parted/parted.h>
 #include <parted/debug.h>
 #include <parted/endian.h>
 
+#ifndef MAX
+# define MAX(a,b) ((a) < (b) ? (b) : (a))
+#endif
+
 #if ENABLE_NLS
 #  include <libintl.h>
 #  define _(String) dgettext (PACKAGE, String)
@@ -82,21 +83,6 @@ _amiga_block_id (uint32_t id) {
 			return "<unknown>";
 	}
 }
-static int
-_amiga_valid_block_id (uint32_t id) {
-	switch (id) {
-		case IDNAME_RIGIDDISK :
-		case IDNAME_BADBLOCK :
-		case IDNAME_PARTITION :
-		case IDNAME_FILESYSHEADER :
-		case IDNAME_LOADSEG :
-		case IDNAME_BOOT :
-			return 1;
-		case IDNAME_FREE :
-		default :
-			return 0;
-	}
-}
 
 struct AmigaIds {
 	uint32_t ID;
@@ -356,7 +342,6 @@ static PedDisk*
 amiga_alloc (const PedDevice* dev)
 {
 	PedDisk *disk;
-	struct AmigaDisk *adsk;
 	struct RigidDiskBlock *rdb;
 	PedSector cyl_size;
 	int highest_cylinder, highest_block;
@@ -505,7 +490,8 @@ amiga_read (PedDisk* disk)
 
 	PED_ASSERT(disk != NULL, return 0);
 	PED_ASSERT(disk->dev != NULL, return 0);
-	PED_ASSERT(disk->dev->sector_size % 512 == 0, return 0);
+	PED_ASSERT(disk->dev->sector_size % PED_SECTOR_SIZE_DEFAULT == 0,
+                   return 0);
 	PED_ASSERT(disk->disk_specific != NULL, return 0);
 	rdb = RDSK(disk->disk_specific);
 
@@ -539,9 +525,7 @@ amiga_read (PedDisk* disk)
 	{
 		PedPartition *part;
 		PedSector start, end;
-		struct DosEnvec *de;
 		PedConstraint *constraint_exact;
-		int j; 
 
 		/* Let's look for loops in the partition table */
 		if (_amiga_loop_check(partblock, partlist, i)) {
@@ -581,11 +565,12 @@ amiga_read (PedDisk* disk)
 		}
 		ped_constraint_destroy (constraint_exact);
 	}
+	ped_free(partition);
 	return 1;
 }
 
 static int
-_amiga_find_free_blocks(PedDisk *disk, uint32_t *table,
+_amiga_find_free_blocks(const PedDisk *disk, uint32_t *table,
 	struct LinkedBlock *block, uint32_t first, uint32_t type)
 {
 	PedSector next;
@@ -643,7 +628,7 @@ _amiga_next_free_block(uint32_t *table, uint32_t start, uint32_t type) {
 	return i;
 }
 static PedPartition *
-_amiga_next_real_partition(PedDisk *disk, PedPartition *part) {
+_amiga_next_real_partition(const PedDisk *disk, PedPartition *part) {
 	PedPartition *next;
 
 	for (next = ped_disk_next_partition (disk, part);
@@ -653,7 +638,7 @@ _amiga_next_real_partition(PedDisk *disk, PedPartition *part) {
 }
 #ifndef DISCOVER_ONLY
 static int
-amiga_write (PedDisk* disk)
+amiga_write (const PedDisk* disk)
 {
 	struct RigidDiskBlock *rdb;
 	struct LinkedBlock *block;
@@ -661,7 +646,7 @@ amiga_write (PedDisk* disk)
 	PedPartition *part, *next_part;
 	PedSector cylblocks, first_hb, last_hb, last_used_hb;
 	uint32_t * table;
-	uint32_t i, rdb_block, max_part;
+	uint32_t i;
 	uint32_t rdb_num, part_num, block_num, next_num;
 
 	PED_ASSERT (disk != NULL, return 0);
@@ -674,6 +659,11 @@ amiga_write (PedDisk* disk)
 	/* Let's read the rdb */
 	if ((rdb_num = _amiga_find_rdb (disk->dev, rdb)) == AMIGA_RDB_NOT_FOUND) {
 		rdb_num = 2;
+		size_t pb_size = sizeof (struct PartitionBlock);
+                /* Initialize only the part that won't be copied over
+                   with a partition block in amiga_read.  */
+		memset ((char *)(RDSK(disk->disk_specific)) + pb_size,
+			0, PED_SECTOR_SIZE_DEFAULT - pb_size);
 	} else {
 		memcpy (RDSK(disk->disk_specific), rdb, PED_SECTOR_SIZE_DEFAULT);
 	}
@@ -686,13 +676,19 @@ amiga_write (PedDisk* disk)
 	last_hb = (PedSector) PED_BE32_TO_CPU (rdb->rdb_RDBBlocksHi);
 	last_used_hb = (PedSector) PED_BE32_TO_CPU (rdb->rdb_HighRDSKBlock);
 
-	/* let's allocate a free block table and initialize it */
-	if (!(table = ped_malloc ((last_hb - first_hb + 1) * sizeof(uint32_t))))
+	/* Allocate a free block table and initialize it.
+	   There must be room for at least RDB_NUM + 2 entries, since
+	   the first RDB_NUM+1 entries get IDNAME_RIGIDDISK, and the
+	   following one must have LINK_END to serve as sentinel.  */
+	size_t tab_size = 2 + MAX (last_hb - first_hb, rdb_num);
+	if (!(table = ped_malloc (tab_size * sizeof *table)))
 		return 0;
 
-	memset(table, 0xff, (last_hb - first_hb + 1) * sizeof(uint32_t));
-	for (i = 0; i<=rdb_num; i++) table[i] = IDNAME_RIGIDDISK;
-	
+	for (i = 0; i <= rdb_num; i++)
+		table[i] = IDNAME_RIGIDDISK;
+	for (     ; i < tab_size; i++)
+		table[i] = LINK_END;
+
 	/* Let's allocate a partition block */
 	if (!(block = ped_malloc (PED_SECTOR_SIZE_DEFAULT))) {
 		ped_free (table);
@@ -781,9 +777,7 @@ amiga_write (PedDisk* disk)
 
 error_free_table:
 	ped_free (table);
-error_free_block:
 	ped_free (block);
-error:
 	return 0;
 }
 #endif /* !DISCOVER_ONLY */
@@ -1033,7 +1027,6 @@ _amiga_get_constraint (const PedDisk *disk)
 	PedDevice *dev = disk->dev;
 	PedAlignment start_align, end_align;
 	PedGeometry max_geom;
-	struct RigidDiskBlock *rdb = RDSK(disk->disk_specific);
 	PedSector cyl_size = dev->hw_geom.sectors * dev->hw_geom.heads;
 
 	if (!ped_alignment_init(&start_align, 0, cyl_size))
@@ -1098,7 +1091,6 @@ amiga_alloc_metadata (PedDisk* disk)
 {
 	PedPartition*		new_part;
 	PedConstraint*		constraint_any = NULL;
-	PedSector		highest_block;
 
 	PED_ASSERT (disk != NULL, goto error);
 	PED_ASSERT (disk->dev != NULL, goto error);
@@ -1175,17 +1167,17 @@ static PedDiskType amiga_disk_type = {
 void
 ped_disk_amiga_init ()
 {
-	PED_ASSERT(sizeof(struct AmigaBlock) != 3, return);
-	PED_ASSERT(sizeof(struct RigidDiskBlock) != 64, return);
-	PED_ASSERT(sizeof(struct PartitionBlock) != 64, return);
-	PED_ASSERT(sizeof(struct LinkedBlock) != 5, return);
-	PED_ASSERT(sizeof(struct Linked2Block) != 18, return);
+	PED_ASSERT (sizeof (struct AmigaBlock) != 3, return);
+	PED_ASSERT (sizeof (struct RigidDiskBlock) != 64, return);
+	PED_ASSERT (sizeof (struct PartitionBlock) != 64, return);
+	PED_ASSERT (sizeof (struct LinkedBlock) != 5, return);
+	PED_ASSERT (sizeof (struct Linked2Block) != 18, return);
 
-	ped_register_disk_type (&amiga_disk_type);
+	ped_disk_type_register (&amiga_disk_type);
 }
 
 void
 ped_disk_amiga_done ()
 {
-	ped_unregister_disk_type (&amiga_disk_type);
+	ped_disk_type_unregister (&amiga_disk_type);
 }
