@@ -7,7 +7,7 @@
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -16,40 +16,36 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 
+#include <config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 
 #include <assert.h>
+#include <wchar.h>
+#include <string.h>
 
-#include <config.h>
+#include "xalloc.h"
+#include "strlist.h"
 
 #ifdef ENABLE_NLS
-#       include <wchar.h>
-        int wcswidth (const wchar_t *s, size_t n);
 #	define L_(str) L##str
 #else
 #	define L_(str) str
 #       ifdef wchar_t
 #               undef wchar_t
 #       endif
-#       include <string.h>
 #       define wchar_t char
 #       define wcslen strlen
 #       define wcswidth strnlen
 #       define wcscat strcat
-#       define wcsdup strdup
-        size_t strnlen (const char *, size_t);
+#       define wcsdup xstrdup
 #endif
 
-#include "xalloc.h"
-#include "strlist.h"
-
 
 static const unsigned int       MAX_WIDTH = 512;
 static const wchar_t*           DELIMITER = L_("  ");
@@ -68,7 +64,7 @@ Table* table_new(int ncols)
 {
         assert ( ncols >= 0 );
         
-        Table *t = malloc(sizeof(Table));
+        Table *t = xmalloc (sizeof(*t));
 
         t->ncols = ncols;
         t->nrows = 0;
@@ -117,7 +113,7 @@ static void table_calc_column_widths (Table* t)
         assert(t->ncols > 0);
         
         if (!t->widths)
-                t->widths = (int*)malloc(t->ncols * sizeof(int));
+                t->widths = xmalloc (t->ncols * sizeof(t->widths[0]));
 
         for (c = 0; c < t->ncols; ++c)
                 t->widths[c] = 0;
@@ -147,8 +143,7 @@ void table_add_row (Table* t, wchar_t** row)
                 printf("[%s]", row[i]);
         putchar ('\n');*/
 
-        t->rows = (wchar_t***)realloc (t->rows, (t->nrows + 1)
-                                                * sizeof(wchar_t***));
+        t->rows = xrealloc (t->rows, (t->nrows + 1) * sizeof(wchar_t***));
          
         t->rows[t->nrows] = row;
 
@@ -160,13 +155,15 @@ void table_add_row (Table* t, wchar_t** row)
 
 void table_add_row_from_strlist (Table* t, StrList* list)
 {
-        wchar_t** row = (wchar_t**)malloc(str_list_length(list)
-                                          * sizeof(wchar_t**));
+        wchar_t** row = xmalloc (str_list_length(list) * sizeof(*row));
         int i = 0;
 
         while (list)
         {
-                row[i] = (wchar_t*)list->str;
+                row[i] = wcsdup (list->str);
+                if (row[i] == NULL)
+                        xalloc_die ();
+
 
                 list = list->next;
                 ++i;
@@ -199,7 +196,7 @@ static void table_render_row (Table* t, int rownum, int ncols, wchar_t** s)
                 int j;
                 int nspaces = max(t->widths[i] - wcswidth(row[i], MAX_WIDTH),
                                   0);
-                wchar_t* pad = xmalloc ((nspaces + 1) * sizeof(wchar_t));
+                wchar_t* pad = xmalloc ((nspaces + 1) * sizeof(*pad));
 
                 for (j = 0; j < nspaces; ++j)
                        pad[j] = L' '; 
@@ -238,7 +235,7 @@ static void table_render_rows (Table* t, wchar_t** s)
  */
 wchar_t* table_render(Table* t)
 {
-        wchar_t* s = malloc(sizeof(wchar_t));
+        wchar_t* s = xmalloc (sizeof(*s));
 
         *s = L_('\0');
         table_render_rows (t, &s);
