@@ -4,7 +4,7 @@
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,8 +13,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /* it's best to compile this with:
@@ -57,7 +56,10 @@
 
 static struct option const long_options[] =
   {
+    /* Note: the --no-update option is deprecated, and deliberately
+     * not documented.  FIXME: remove --no-update in 2009. */
     {"no-update", no_argument, NULL, 'd'},
+    {"dry-run", no_argument, NULL, 'd'},
     {"summary", no_argument, NULL, 's'},
     {"help", no_argument, NULL, 'h'},
     {"version", no_argument, NULL, 'v'},
@@ -68,7 +70,7 @@ static struct option const long_options[] =
 char *program_name;
 
 /* initialized to 0 according to the language lawyers */
-static int	opt_no_probe;
+static int	opt_no_inform;
 static int	opt_summary;
 
 static void
@@ -115,7 +117,7 @@ process_dev (PedDevice* dev)
 	disk = ped_disk_new (dev);
 	if (!disk)
 		goto error;
-	if (!opt_no_probe) {
+	if (!opt_no_inform) {
 		if (!ped_disk_commit_to_os (disk))
 			goto error_destroy_disk;
 	}
@@ -140,16 +142,16 @@ usage (int status)
     {
       printf (_("Usage: %s [OPTION] [DEVICE]...\n"), PROGRAM_NAME);
       fputs (_("\
-Inform the OS of partition table changes.\n\
+Inform the operating system about partition table changes.\n\
 \n\
-  -d, --no-update  don't update the kernel\n\
+  -d, --dry-run    do not actually inform the operating system\n\
   -s, --summary    print a summary of contents\n\
   -h, --help       display this help and exit\n\
   -v, --version    output version information and exit\n\
 "), stdout);
       fputs (_("\
 \n\
-With no DEVICE, probe all partitions.\n\
+When no DEVICE is given, probe all partitions.\n\
 "), stdout);
       printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
@@ -168,7 +170,7 @@ main (int argc, char* argv[])
 	while ((c = getopt_long (argc, argv, "dhsv", long_options, NULL)) != -1)
 		switch (c) {
 			case 'd':
-				opt_no_probe = 1;
+				opt_no_inform = 1;
 				break;
 
 			case 's':
